open Latex;;

open Melt;;

# 1 "../manual.mlt"
open Latex.Verbatim

let regstab = (mode T (((text "Reg") ^^ (textsc (mode T ((text "stab")))))))
let sat = textsc (mode T ((text "sat")))
let dpll = textsc (mode T ((text "dpll")))
let onearg_cmd cmd arg = command cmd [T,arg] T
let graphicx_cmd = command ~packages:["graphicx",""]
let dots = command "dots" [] T
let href_cmd = command ~packages:["hyperref",""]
let prefixed ?(prefix=(mode T ((symbolc '<' ^^ ((text "PREFIX>")))))) path = texttt (mode T ((prefix ^^ (if path=(mode T ((text ""))) then (mode T ((text ""))) else (mode T (((text "/") ^^ (path))))))))
let file = texttt
let cmd = texttt
let manual_file = file (mode T ((text "manual.pdf")))
let vim_file = file (mode T ((text "regstab.vim")))
let regstab_byte = file (mode T ((text "regstab")))
let regstab_opt = file (mode T ((text "regstab.opt")))
let sch2cnf_byte = file (mode T ((text "schf2cnf")))
let sch2cnf_opt = file (mode T ((text "schf2cnf.opt")))

let lbl_lg_prop = label()

let includegraphics ?scale src = 
  match scale with
  |None   -> includegraphics src
  |Some s -> graphicx_cmd "includegraphics" ~opt:(T,(mode T (((text "scale=") ^^ (s))))) [T,src] T

let url u = href_cmd "url" [T,u] T
let footurl u = footnote (url u)
let mail m = href_cmd "href" [T,(mode T (((text "mailto:") ^^ (m)))); T,m] T
let href url label = href_cmd "href" [T,url; T,label] T

let description ts =
  environment "description" (T,(concat (StdLabels.List.map ts ~f:(fun (t1,t2) -> (mode T (command "item" ~opt:(T,t1) [T,t2] T)))))) T

let title = (mode T ((includegraphics ~scale:(mode T ((text "0.5"))) (mode T ((text "RegSTABLogo.png"))) ^^ (newline) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (regstab) ^^ (newline) ^^ (small (url (mode T ((text "http://regstab.forge.ocamlcore.org/"))))))))

let author =
    (mode T (((text "Vincent Aravantinos ") ^^ (newline) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (small (mail (mode T ((text "vincent.aravantinos@gmail.com"))))) ^^ ((text " ")) ^^ (newline) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (small (url (mode T ((text "http://membres-liglab.imag.fr/aravantinos"))))))))

let cite = onearg_cmd "cite"
let bibliographystyle = onearg_cmd "bibliographystyle"
let bibliography = onearg_cmd "bibliography"

let conj   = cmd (verbatim "/\\")
let disj   = cmd (verbatim "\\/")
let xor    = cmd (verbatim "(+)")
let equiv  = cmd (verbatim "<->")
let imply  = cmd (verbatim "->")
let neg    = cmd (verbatim "~")
let iteration ~op ?(var=(mode M ((text "i")))) ?(cstr : [`Itvl of Latex.t * Latex.t | `Cstr of Latex.t] =`Itvl((mode M ((text "1"))),(mode M ((text "n"))))) sch =
  match cstr with
  |`Cstr c -> (mode M ((index op (mode M ((var ^^ ((text "|")) ^^ (c)))) ^^ ((text " ")) ^^ (sch))))
  |`Itvl(a,b) -> (mode M ((op ^^ ((text "_")) ^^ (block (mode M ((var ^^ ((text "=")) ^^ (a))))) ^^ ((text "^")) ^^ (b) ^^ ((text " ")) ^^ (sch))))
let it_and = iteration ~op:bigwedge
let it_or  = iteration ~op:bigvee

let doc = ((mode T (((text "\n")
 ^^ ((text "  ")) ^^ (tableofcontents) ^^ ((text "\n")
) ^^ ((text "  ")) ^^ (section (mode T ((text "Description")))) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (regstab) ^^ ((text " is a ")) ^^ (sat) ^^ ((text "-solver extended to handle formula schemata")) ^^ ((text "\n")
) ^^ ((text "    i.e. constructions of the form ")) ^^ ((mode M (it_and (mode M ((lnot ^^ ((text " P_i")) ^^ (lor_) ^^ (index (mode M ((text "P"))) (mode M ((text "i+1")))))))))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "    Such schemata are considered to be unsatisfiable iff all propositional formulae of the corresponding form are unsatisfiable.")) ^^ ((par)

) ^^ ((text "    It is generally not possible to automatize the (un)satisfiability of such objects.")) ^^ ((text "\n")
) ^^ ((text "    So ")) ^^ (regstab) ^^ ((text " is restricted to a specific form of schemata called ``regular schemata''.")) ^^ ((text "\n")
) ^^ ((text "    Hence the part ``Reg'' of ")) ^^ (regstab) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "    Furthermore ")) ^^ (regstab) ^^ ((text " is based on an extension of propositional tableaux called ")) ^^ (textsc (mode T ((text "stab")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "    Hence the part ``")) ^^ (textsc (mode T ((text "stab")))) ^^ ((text "'' of ")) ^^ (regstab) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "    This is all described in detail in ")) ^^ (cite (mode T ((text "tab09")))) ^^ ((text ".")) ^^ ((par)

) ^^ ((text "    It is quite unusual to use propositional tableaux for a ")) ^^ (sat) ^^ ((text "-solver ")) ^^ ((text "\n")
) ^^ ((text "    but this is much more natural to use tableaux rather than ")) ^^ (dpll) ^^ ((text " to handle schemata ")) ^^ (cite (mode T ((text "sd09")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "    As a pure ")) ^^ (sat) ^^ ((text "-solver ")) ^^ (regstab) ^^ ((text " is all the least efficient.")) ^^ ((text "\n")
) ^^ ((text "    But one can easily think of combining ")) ^^ (regstab) ^^ ((text " with an efficient ")) ^^ (sat) ^^ ((text "-solver in order to benefit of both worlds.")) ^^ ((par)

) ^^ ((text "  ")) ^^ (section (mode T ((text "Install")))) ^^ ((text "\n")
) ^^ ((text "    Preamble: install on Windows seems rather painful, if you agree then")) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (href (mode T (((text "http://forge.ocamlcore.org/tracker/index.php?func=detail&aid=285&group") ^^ ((text "\\_")) ^^ ((text "id=111&atid=548"))))) (mode T ((text "complain!")))) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "GODI")))) ^^ ((text "\n")
) ^^ ((text "      The simplest way to install ")) ^^ (regstab) ^^ ((text " is to use GODI")) ^^ (footurl (mode T ((text "http://godi.camlcity.org/godi/index.html")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      GODI is a package manager for Ocaml libraries and software.")) ^^ ((text "\n")
) ^^ ((text "      It has many many advantages for Ocaml apps developpers.")) ^^ ((par)

) ^^ ((text "      Currently, the official version of GODI relies on ocaml 3.10,")) ^^ ((text "\n")
) ^^ ((text "      there is a beta version of GODI for ocaml 3.11.1")) ^^ (footurl (mode T ((text "http://download.camlcity.org/download/godi-rocketboost-20090421.tar.gz")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      See GODI documentation and install the package ")) ^^ ((text "\"")) ^^ ((text "apps-regstab")) ^^ ((text "\"")) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      The following will be installed (")) ^^ (prefixed (mode T ((text "")))) ^^ ((text " is GODI base directory):")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T ((regstab_byte ^^ ((text ", ")) ^^ (regstab_opt) ^^ ((text " (if any), ")) ^^ (sch2cnf_byte) ^^ ((text ", ")) ^^ (sch2cnf_opt) ^^ ((text " in ")) ^^ (prefixed (mode T ((text "bin")))) ^^ ((text ".")))));
        (mode T (((text "The manual (this file) into ") ^^ (prefixed (mode T ((text "doc/apps-regstab/")))) ^^ ((text ".")))));
        (mode T (((text "The man pages into the directory ") ^^ (prefixed (mode T ((text "man/man1/")))) ^^ ((text ".")))));
        (mode T (((text "The developer doc into the directory ") ^^ (prefixed (mode T (((text "share/apps-regstab/developper") ^^ ((text "\\_")) ^^ ((text "doc")))))) ^^ ((text ".")))));
        (mode T (((text "The vim syntax file into the directories ") ^^ (prefixed ~prefix:(mode T (((text "\\$") ^^ ((text "HOME"))))) (mode T ((text ".vim/syntax")))) ^^ ((text " and ")) ^^ (prefixed (mode T ((text "share/apps-regstab/vim")))) ^^ ((text ".")))));
      ]) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Good Old Make")))) ^^ ((text "\n")
) ^^ ((text "      Steps:")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (enumerate [
        (mode T ((cmd (mode T ((text "make all"))) ^^ ((par)

) ^^ ((text "          Compile the byte-code version of ")) ^^ (regstab) ^^ ((text ".")))));

        (mode T (((text "(Optional) ") ^^ (cmd (mode T ((text "make opt")))) ^^ ((par)

) ^^ ((text "          Compile the native-code version of ")) ^^ (regstab) ^^ ((text " if possible on your machine (")) ^^ ((mode M (rightarrow))) ^^ ((text " executable ")) ^^ (regstab_opt) ^^ ((text ").")))));

        (mode T (((text "(Optional) ") ^^ (cmd (mode T ((text "make test")))) ^^ ((par)

) ^^ ((text "          Run tests.")))));

        (mode T (cmd (mode T ((text "make install"))))) ^^
          let prefixed' = prefixed ~prefix:(mode T (((text "\\$") ^^ ((text "PREFIX"))))) in
                (mode T ((itemize [
                  (mode T (((text "Copy the files ") ^^ (regstab_byte) ^^ ((text ", ")) ^^ (regstab_opt) ^^ ((text " (if any), ")) ^^ (sch2cnf_byte) ^^ ((text ", ")) ^^ (sch2cnf_opt) ^^ ((text " into the directory ")) ^^ (prefixed' (mode T ((text "bin")))) ^^ ((text ".")))));
                  (mode T (((text "Copy the manual (this file) into the directory ") ^^ (prefixed' (mode T ((text "doc/regstab/")))) ^^ ((text ".")))));
                  (mode T (((text "Copy the man pages into the directory ") ^^ (prefixed' (mode T ((text "man/man1/")))) ^^ ((text ".")))));
                  (mode T (((text "Copy the developer doc into the directory ") ^^ (prefixed' (mode T (((text "share/regstab/developper") ^^ ((text "\\_")) ^^ ((text "doc")))))) ^^ ((text ".")))));
                  (mode T (((text "Copy the vim syntax file into the directories ") ^^ (prefixed ~prefix:(mode T (((text "\\$") ^^ ((text "HOME"))))) (mode T ((text ".vim/syntax")))) ^^ ((text " and ")) ^^ (prefixed' (mode T ((text "share/regstab/vim")))) ^^ ((text ".")))));
                ] ^^ ((text "\n")
) ^^ ((text "              The environment variable ")) ^^ (file (mode T ((text "PREFIX")))) ^^ ((text " defaults to ")) ^^ (file (mode T ((text "/usr/local")))) ^^ ((text ".")))));
        ]) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (paragraph (mode T ((text "Dependencies.")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (enumerate [
        (mode T (((text "the Ocaml compiler, tested with 3.11.1") ^^ (footurl (mode T ((text "http://caml.inria.fr/index.en.html")))))));
        (mode T (((text "the Findlib library manager, tested with 1.2.4") ^^ (footurl (mode T ((text "http://projects.camlcity.org/projects/findlib.html/")))))));
        (mode T (((text "the OMake build system, tested with 0.9.8.5") ^^ (footurl (mode T ((text "http://omake.metaprl.org/index.html")))))));
        (mode T (((text "the Jane Street Core library, tested with 0.5.3") ^^ (footurl (mode T ((text "http://ocaml.janestreet.com/?q=node/13")))))));
      ]) ^^ ((text "\n")
) ^^ ((text "      And possibly:")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T (((text "If you're under Windows: CygWin") ^^ (footurl (mode T ((text "http://www.cygwin.com/")))) ^^ ((text ".")))));
        (mode T (((text "If you want to run tests: OUnit") ^^ (footurl (mode T ((text "http://www.xs4all.nl/~mmzeeman/ocaml/")))))));
      ]) ^^ ((text "\n")
) ^^ ((text "      Notice that all those dependencies, except CygWin, can be handled through GODI.")) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Intel Mac OSX Binaries")))) ^^ ((text "\n")
) ^^ ((text "      The Intel Mac OSX archive contains the following:")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T ((regstab_opt ^^ ((text ":      Intel Mac OSX native executable")))));
        (mode T ((regstab_byte ^^ ((text ":     Machine-independent bytecode executable")))));
        (mode T ((file (mode T ((text "bin/"))) ^^ ((text ":      Contains ")) ^^ (sch2cnf_byte) ^^ ((text ", ")) ^^ (sch2cnf_opt) ^^ ((text " and copies of ")) ^^ (regstab_byte) ^^ ((text " and ")) ^^ (regstab_opt))));
        (mode T ((file (mode T ((text "doc/"))) ^^ ((text ":      Contains the manual ")) ^^ (manual_file) ^^ ((text " (this file)")))));
        (mode T ((file (mode T ((text "examples/"))) ^^ ((text ": Contains examples")))));
        (mode T ((file (mode T ((text "man/man1/"))) ^^ ((text ": Contains the man pages for ")) ^^ (regstab_byte) ^^ ((text ", ")) ^^ (regstab_opt) ^^ ((text ", ")) ^^ (sch2cnf_byte) ^^ ((text ", ")) ^^ (sch2cnf_opt))));
        (mode T ((file (mode T ((text "tools/"))) ^^ ((text ":    Contains ")) ^^ (vim_file) ^^ ((text " the vim syntax file for ")) ^^ (regstab) ^^ ((text " files")))));
      ]) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Machine-Independant Bytecode")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (emph (mode T (((text "Warning: the bytecode version of ") ^^ (regstab) ^^ ((text " is much slower than the native one ")) ^^ ((text "\n")
) ^^ ((text "      (")) ^^ ((mode M ((text "2s")))) ^^ ((text " vs. ")) ^^ ((mode M ((text "30s")))) ^^ ((text " for ")) ^^ (file (mode T ((text "examples/adder4.stab")))) ^^ ((text " on my machine).")) ^^ ((text "\n")
) ^^ ((text "      Though you may have no other choice: e.g. if your architecture is not supported by the OCaml native compiler (very rare)")) ^^ ((text "\n")
) ^^ ((text "      or by one of the dependencies (e.g. Windows without Cygwin)")))))) ^^ ((par)

) ^^ ((text "      The Bytecode archive contains the following:")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T ((regstab_byte ^^ ((text ":     Machine-independent bytecode executable")))));
        (mode T ((file (mode T ((text "bin/"))) ^^ ((text ":      Contains ")) ^^ (sch2cnf_byte) ^^ ((text " and a copy of ")) ^^ (regstab_byte))));
        (mode T ((file (mode T ((text "doc/"))) ^^ ((text ":      Contains the manual ")) ^^ (manual_file) ^^ ((text " (this file)")))));
        (mode T ((file (mode T ((text "examples/"))) ^^ ((text ": Contains examples")))));
        (mode T ((file (mode T ((text "man/man1/"))) ^^ ((text ": Contains the man pages for ")) ^^ (regstab_byte) ^^ ((text " and ")) ^^ (sch2cnf_byte))));
        (mode T ((file (mode T ((text "tools/"))) ^^ ((text ":    Contains ")) ^^ (vim_file) ^^ ((text " the vim syntax file for ")) ^^ (regstab) ^^ ((text " files")))));
      ]) ^^ ((par)

) ^^ ((text "  ")) ^^ (section (mode T ((text "Usage")))) ^^ ((par)

) ^^ ((text "    ")) ^^ (regstab_opt) ^^ ((text " ")) ^^ ((mode M ((text "[")))) ^^ ((text "-verbose")) ^^ ((mode M ((text "]")))) ^^ ((text " ")) ^^ ((mode M ((text "[file]")))) ^^ ((par)

) ^^ ((text "    ")) ^^ (noindent) ^^ (regstab_byte) ^^ ((text " ")) ^^ ((mode M ((text "[")))) ^^ ((text "-verbose")) ^^ ((mode M ((text "]")))) ^^ ((text " ")) ^^ ((mode M ((text "[file]")))) ^^ ((par)

) ^^ ((text "    ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "    Prints ")) ^^ (cmd (mode T ((text "unsatisfiable")))) ^^ ((text " (resp. ")) ^^ (cmd (mode T ((text "satisfiable")))) ^^ ((text ") if the input formula is unsatisfiable (resp. satisfiable).")) ^^ ((text "\n")
) ^^ ((text "    If no file is provided the input formula is taken on ")) ^^ (cmd (mode T ((text "stdin")))) ^^ ((text ".")) ^^ ((par)

) ^^ ((text "    ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (paragraph (mode T ((text "Options:")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (description [
        (mode T (cmd (mode T ((text "-verbose"))))), (mode T (((text "Be verbose, currently only displays the input formula after parsing.") ^^ ((text "\n")
) ^^ ((text "        Allows to check that the input formula is indeed the formula understood by ")) ^^ (regstab) ^^ ((text ".")))));
      ]) ^^ ((text "\n")
) ^^ ((text "  ")) ^^ (section (mode T ((text "(Informal) Language Definition")))) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (subsection ~label:lbl_lg_prop (mode T ((text "Propositional Formulae")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T (((text "Usual logical notations are translated into ASCII:") ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (conj) ^^ ((text " stands for the conjunction (")) ^^ ((mode M (land_))) ^^ ((text "),")) ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (disj) ^^ ((text " stands for the disjunction (")) ^^ ((mode M (lor_))) ^^ ((text "),")) ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (neg) ^^ ((text " stands for the negation (")) ^^ ((mode M (lnot))) ^^ ((text ").")) ^^ ((par)

) ^^ ((text "        As a convenience some other usual connectives are pre-defined:")) ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1")) ^^ (imply) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "2")))))) ^^ ((text " stands for the implication (")) ^^ ((mode M (((text "P_1") ^^ (rightarrow_) ^^ ((text " P_2 := ")) ^^ (lnot) ^^ ((text "P_1")) ^^ (lor_) ^^ ((text "P_2")))))) ^^ ((text "),")) ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1")) ^^ (equiv) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "2")))))) ^^ ((text " stands for the equivalence (")) ^^ ((mode M (((text "P_1") ^^ (leftrightarrow_) ^^ ((text " P_2 := (P_1")) ^^ (rightarrow_) ^^ ((text "P_2)")) ^^ (land_) ^^ ((text "(P_2")) ^^ (rightarrow_) ^^ ((text "P_1)")))))) ^^ ((text "),")) ^^ ((text "\n")
) ^^ ((text "          ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1")) ^^ (xor) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "2")))))) ^^ ((text " stands for the exclusive or (")) ^^ ((mode M (((text "P_1") ^^ (oplus) ^^ ((text " P_2 := ")) ^^ (lnot) ^^ ((text "(P_1")) ^^ (leftrightarrow_) ^^ ((text "P_2)")))))) ^^ ((text ").")))));

        (mode T (((text "Propositional variables must be indexed:") ^^ ((text "\n")
) ^^ ((text "          you can't write ")) ^^ (cmd (mode T (((text "A") ^^ (conj) ^^ ((text "(B")) ^^ (disj) ^^ ((text "C)")))))) ^^ ((text " but ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1")) ^^ (conj) ^^ ((text "(P")) ^^ ((text "\\_")) ^^ ((text "2")) ^^ (disj) ^^ ((text "Q")) ^^ ((text "\\_")) ^^ ((text "1)")))))) ^^ ((text " is ok.")) ^^ ((text "\n")
) ^^ ((text "        They can be any alphanumerical sequence starting with an uppercase letter.")) ^^ ((text "\n")
) ^^ ((text "        Prime (")) ^^ (cmd (mode T ((text "'")))) ^^ ((text ") can be appended to the sequence.")) ^^ ((text "\n")
) ^^ ((text "        The index may be any integer.")))));

        (mode T (((text "Formulae must be in negative normal form i.e. negation can only occur just in front of a propositional variable:") ^^ ((text "\n")
) ^^ ((text "          you can't write ")) ^^ (cmd (mode T ((neg ^^ ((text "(P")) ^^ ((text "\\_")) ^^ ((text "1")) ^^ (conj) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "2)")))))) ^^ ((text " but ")) ^^ (cmd (mode T ((neg ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "1")) ^^ (disj) ^^ (neg) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "2")))))) ^^ ((text " is ok.")))));

        (mode T (((text "Precedence of connectives is as follows: ") ^^ (conj) ^^ ((text " ")) ^^ ((mode M ((text ">")))) ^^ ((text " ")) ^^ (disj) ^^ ((text " ")) ^^ ((mode M ((text ">")))) ^^ ((text " ")) ^^ (xor) ^^ ((text " ")) ^^ ((mode M ((text ">")))) ^^ ((text " ")) ^^ (equiv) ^^ ((text ",")) ^^ (imply) ^^ ((text ".")))));
      ]) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Schemata")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (paragraph (mode T ((text "Syntax:")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T (((text "Iterated conjunctions are written ``") ^^ (cmd (mode T ((conj ^^ ((mode M ((text "i")))) ^^ ((text "=")) ^^ ((mode M ((text "k")))) ^^ ((text "..")) ^^ ((mode M ((text "e")))))))) ^^ ((text "''")) ^^ ((text "\n")
) ^^ ((text "        where ")) ^^ ((mode M ((text "i")))) ^^ ((text " is a variable, ")) ^^ ((mode M ((text "k")))) ^^ ((text " is an integer, and ")) ^^ ((mode M ((text "e")))) ^^ ((text " is an arithmetic expression.")) ^^ ((text "\n")
) ^^ ((text "        ")) ^^ ((mode M ((text "k")))) ^^ ((text " is called the ")) ^^ (emph (mode T ((text "lower bound")))) ^^ ((text " of the iterated conjunction,")) ^^ ((text "\n")
) ^^ ((text "        ")) ^^ ((mode M ((text "e")))) ^^ ((text " is its ")) ^^ (emph (mode T ((text "upper bound")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "        Iterated disjunctions are written similarly with ")) ^^ (disj) ^^ ((text " instead of ")) ^^ (conj) ^^ ((text ".")))));

        (mode T (((text "Arithmetic expressions are written ``") ^^ (cmd (mode T (((mode M ((text "n"))) ^^ ((text "+")) ^^ ((mode M ((text "k")))))))) ^^ ((text "'' or ``")) ^^ (cmd (mode T (((mode M ((text "n"))) ^^ ((text "-")) ^^ ((mode M ((text "k")))))))) ^^ ((text "'' where ")) ^^ ((mode M ((text "n")))) ^^ ((text " is a variable and ")) ^^ ((mode M ((text "k")))) ^^ ((text " is a natural number.")))));

        (mode T (((text "Inside iterations indexed propositional variables are written ``") ^^ (cmd (mode T (((mode M ((text "P"))) ^^ ((text "\\_")) ^^ ((mode M ((text "e")))))))) ^^ ((text "'' where ")) ^^ ((mode M ((text "P")))) ^^ ((text " is a propositional variable ")) ^^ ((text "\n")
) ^^ ((text "        (defined in Section ~")) ^^ (ref_ lbl_lg_prop) ^^ ((text ") and ")) ^^ ((mode M ((text "e")))) ^^ ((text " is an arithmetic expression.")) ^^ ((text "\n")
) ^^ ((text "        ")) ^^ (emph (mode T (((text "Do not put parentheses around ") ^^ ((mode M ((text "e")))))))) ^^ ((text ".")))));

        (mode T (((text "Variables can be any alphanumerical sequence starting with a lower case letter.") ^^ ((text "\n")
) ^^ ((text "        Prime (")) ^^ (cmd (mode T ((text "'")))) ^^ ((text ") can be appended to the sequence.")))));

        (mode T (((text "Iteration operators have the highest precedence:") ^^ ((text "\n")
) ^^ ((text "        ")) ^^ (cmd (mode T ((conj ^^ ((text "i=0..n P")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (conj) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text " is interpreted as ")) ^^ (cmd (mode T (((text "(") ^^ (conj) ^^ ((text "i=0..n P")) ^^ ((text "\\_")) ^^ ((text "i)")) ^^ (conj) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text ",")) ^^ ((text "\n")
) ^^ ((text "        and not ")) ^^ (cmd (mode T ((conj ^^ ((text "i=0..n (P")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (conj) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "i+1)")))))) ^^ ((text "\n")
) ^^ ((text "        (think of the body of the iteration as being an argument given to the operator ")) ^^ (cmd (mode T ((conj ^^ ((text "i=0..n")))))) ^^ ((text ").")))))
      ]) ^^ ((par)

) ^^ ((text "      ")) ^^ (paragraph (mode T ((text "Example:")))) ^^ ((par)

) ^^ ((text "      ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1 ")) ^^ (conj) ^^ ((text " ")) ^^ (conj) ^^ ((text " i=1..n-1 (P")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "i+1) ")) ^^ (conj) ^^ ((text " ")) ^^ (neg) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "n")))))) ^^ ((par)

) ^^ ((text "      ")) ^^ (paragraph (mode T ((text "Restrictions:")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (itemize [
        (mode T ((emph (mode T ((text "Iterations cannot be nested"))) ^^ ((text ":")) ^^ ((text "\n")
) ^^ ((text "          you cannot write ")) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n (")) ^^ (disj) ^^ ((text "j=1..n ")) ^^ (dots) ^^ ((text ")")))))))));

        (mode T ((emph (mode T ((text "There may be only one free variable"))) ^^ ((text " (called the ")) ^^ (emph (mode T ((text "parameter")))) ^^ ((text " of the schema):")) ^^ ((text "\n")
) ^^ ((text "          you cannot write ")) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n P")) ^^ ((text "\\_")) ^^ ((text "i ")) ^^ (conj) ^^ (conj) ^^ ((text "i=2..p Q")) ^^ ((text "\\_")) ^^ ((text "i")))))) ^^ ((text ".")))));

        (mode T (emph (mode T ((text "All iterations must have the same lower bound"))))) ^^
              (let it_and_2_n = it_and ~cstr:(`Itvl((mode M ((text "2"))),(mode M ((text "n"))))) in
              (mode T ((footnote (mode T (((text "This can be easily circumvented ") ^^ (emph (mode T (((text "if ") ^^ ((mode M ((text "n>1")))))))) ^^ ((text " by manually unfolding the first ranks:")) ^^ ((text "\n")
) ^^ ((text "                ")) ^^ ((mode M ((it_and (mode M ((text "S_i"))) ^^ ((text " ")) ^^ (land_) ^^ ((text " ")) ^^ (it_and_2_n (mode M ((text "T_i")))))))) ^^ ((text " is equivalent, ")) ^^ (emph (mode T (((text "if ") ^^ ((mode M ((text "n>1")))))))) ^^ ((text ", to")) ^^ ((text "\n")
) ^^ ((text "                ")) ^^ ((mode M (((text "S_1") ^^ (land_) ^^ (it_and_2_n (mode M ((text "S_i")))) ^^ ((text " ")) ^^ (land_) ^^ ((text " ")) ^^ (it_and_2_n (mode M ((text "T_i"))))))))))) ^^ ((text ":")) ^^ ((text "\n")
) ^^ ((text "                  you cannot write ")) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n ")) ^^ (dots) ^^ (conj) ^^ (conj) ^^ ((text "i=2..n ")) ^^ (dots)))))))));

        (mode T (emph (mode T (((text "For ") ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((mode M ((text "e")))))))) ^^ ((text " occurring in some iteration, the only variable that can occur in ")) ^^ ((mode M ((text "e")))) ^^ ((text " is the variable which is iterated")))))))^^
              (mode T ((footnote (mode T (((text "This can be easily circumvented by factorising the constant indexed proposition:") ^^ ((text "\n")
) ^^ ((text "                ")) ^^ (it_and (mode M (((text "(P_n ") ^^ (lor_) ^^ ((text " P_i)")))))) ^^ ((text " is equivalent to ")) ^^ ((mode M (((text "P_n") ^^ (lor_) ^^ (it_and (mode M ((text "P_i")))))))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "              Maybe we should automatize this."))))) ^^ ((text "\n")
) ^^ ((text "          you cannot write ")) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n P")) ^^ ((text "\\_")) ^^ ((text "n+1")))))) ^^ ((text " but ")) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n P")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text " is ok.")))));
      ]) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Constraints")))) ^^ ((text "\n")
) ^^ ((text "      Basic constraints can be given on the parameter of a schema.")) ^^ ((text "\n")
) ^^ ((text "      They must be inserted after the schema")) ^^ ((text "\n")
) ^^ ((text "      and are written ``")) ^^ (cmd (mode T (((text "| ") ^^ ((mode M ((text "n")))) ^^ ((text " ")) ^^ ((mode M ((text "op")))) ^^ ((text " ")) ^^ ((mode M ((text "k")))))))) ^^ ((text "''")) ^^ ((text "\n")
) ^^ ((text "      where ")) ^^ ((mode M ((text "n")))) ^^ ((text " is the parameter of the schema, ")) ^^ ((mode M ((text "k")))) ^^ ((text " is an integer, and ")) ^^ ((mode M (((text "op") ^^ (in_) ^^ (left `Brace) ^^ (cmd (verbatim "<,<=,=,>=,>")) ^^ (right `Brace))))) ^^ ((par)

) ^^ ((text "      Example:")) ^^ ((par)

) ^^ ((text "        ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1 ")) ^^ (conj) ^^ ((text " ")) ^^ (conj) ^^ ((text "i=1..n-1 (P")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "i+1) ")) ^^ (conj) ^^ (neg) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "n | n > 0")))))) ^^ ((par)

) ^^ ((text "      ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "      Notice that this example is unsatisfiable with the constraint but is satisfiable without it:")) ^^ ((text "\n")
) ^^ ((text "        if we take ")) ^^ ((mode M ((cmd (mode T ((text "n"))) ^^ ((text "=0")))))) ^^ ((text " we get the formula ")) ^^ (cmd (mode T (((text "P") ^^ ((text "\\_")) ^^ ((text "1")) ^^ (conj) ^^ (neg) ^^ ((text "P")) ^^ ((text "\\_")) ^^ ((text "0")))))) ^^ ((text " which is satisfiable.")) ^^ ((text "\n")
) ^^ ((text "        As schemata are considered to be unsatisfiable iff all propositional formulae obtained by giving a value to ")) ^^ (cmd (mode T ((text "n")))) ^^ ((text " are unsatisfiable,")) ^^ ((text "\n")
) ^^ ((text "      this schema is not satisfiable.")) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Functions")))) ^^ ((text "\n")
) ^^ ((text "      To ease the input you can define simple functions.")) ^^ ((text "\n")
) ^^ ((text "      E.g. if you use often ")) ^^ (cmd (mode T (((text "A") ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "A")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text " with a different ")) ^^ (cmd (mode T ((text "A")))) ^^ ((text " (say ")) ^^ (cmd (mode T (((text "B") ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "B")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text ", ")) ^^ (cmd (mode T (((text "C") ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "C")) ^^ ((text "\\_")) ^^ ((text "i+1")))))) ^^ ((text ", ")) ^^ (dots) ^^ ((text "),")) ^^ ((text "\n")
) ^^ ((text "      then you can factorize this by defining a function ")) ^^ ((mode M ((lambda ^^ ((text " X")) ^^ (cdot) ^^ ((text " X_i")) ^^ (rightarrow_) ^^ ((text " X_")) ^^ (block (mode M ((text "i+1")))))))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      The syntax is as follows: ")) ^^ (cmd (mode T (((text "let F(X) := X") ^^ ((text "\\_")) ^^ ((text "i")) ^^ (imply) ^^ ((text "X")) ^^ ((text "\\_")) ^^ ((text "i+1 in ")) ^^ (dots))))) ^^ ((par)

) ^^ ((text "      ")) ^^ (itemize [
        (mode T ((text "The name of a function follows the same conventions as propositional variable names.")));
        (mode T (((text "The parameters of the function is a comma separated list comprised between parentheses if the list is non-empty.") ^^ ((text "\n")
) ^^ ((text "              ")) ^^ (emph (mode T ((text "The parameters may be either propositional variable names or simple variable names.")))) ^^ ((text "\n")
) ^^ ((text "              E.g. you can write ")) ^^ (cmd (mode T (((text "let F(X,n) := X") ^^ ((text "\\_")) ^^ ((text "n ")) ^^ (imply) ^^ ((text " X")) ^^ ((text "\\_")) ^^ ((text "n+1 in ")) ^^ (dots))))))));
        (mode T (((text "The right member of the affectation is any formula as defined previously.") ^^ ((text "\n")
) ^^ ((text "        It cannot contain a constraint.")))));
      ]) ^^ ((par)

) ^^ ((text "      Calling the function is done, e.g., as follows: ")) ^^ (cmd (mode T ((text "F(P,n+1)")))) ^^ ((text ", i.e. the name of the function followed by the list of parameters enclosed between")) ^^ ((text "\n")
) ^^ ((text "      parentheses. ")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (emph (mode T (((text "When there is no parameter, you should still put parentheses, i.e. ") ^^ (cmd (mode T ((text "F()")))))))) ^^ ((text ".")) ^^ ((par)

) ^^ ((text "      Full Example:")) ^^ ((par)

) ^^ ((text "      ")) ^^ (noindent) ^^ (cmd (mode T (((text "let F(S,A,B,C,i) := S") ^^ ((text "\\_")) ^^ ((text "i ")) ^^ (equiv) ^^ ((text " (A")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (xor) ^^ ((text "B")) ^^ ((text "\\_")) ^^ ((text "i")) ^^ (xor) ^^ ((text "C")) ^^ ((text "\\_")) ^^ ((text "i-1) in")))))) ^^ ((par)

) ^^ ((text "      ")) ^^ (noindent) ^^ (cmd (mode T ((conj ^^ ((text "i=1..n (F(S,A,B,C,i) ")) ^^ (disj) ^^ ((text " F(S',A',B',C,i+1))")))))) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Comments")))) ^^ ((text "\n")
) ^^ ((text "      Comments start by ")) ^^ (cmd (mode T ((text "//")))) ^^ ((text " and end at the end of the line.")) ^^ ((par)

) ^^ ((text "  ")) ^^ (section (mode T ((text "Examples")))) ^^ ((text "\n")
) ^^ ((text "    See the directory ")) ^^ (cmd (mode T ((text "examples")))) ^^ ((text ".")) ^^ ((par)

) ^^ ((text "  ")) ^^ (section (mode T ((text "Tools")))) ^^ ((text "\n")
) ^^ ((text "    ")) ^^ (subsection (mode T ((text "sch2cnf")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (sch2cnf_opt) ^^ ((text " ")) ^^ ((mode M ((text "n")))) ^^ ((text " ")) ^^ ((mode M ((text "[file]")))) ^^ ((par)

) ^^ ((text "      ")) ^^ (noindent) ^^ (sch2cnf_byte) ^^ ((text " ")) ^^ ((mode M ((text "n")))) ^^ ((text " ")) ^^ ((mode M ((text "[file]")))) ^^ ((par)

) ^^ ((text "      ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "      Computes the propositional formula obtained by giving the value ")) ^^ ((mode M ((text "n")))) ^^ ((text " to the parameter of the input schema.")) ^^ ((text "\n")
) ^^ ((text "      Outputs the formula in DIMACS cnf format.")) ^^ ((text "\n")
) ^^ ((text "      Thus ")) ^^ (sch2cnf_byte) ^^ ((text " can be used as a generator of problems for ")) ^^ (sat) ^^ ((text "-solvers.")) ^^ ((text "\n")
) ^^ ((text "      If no file is provided the input formula is taken on ")) ^^ (cmd (mode T ((text "stdin")))) ^^ ((text ".")) ^^ ((par)

) ^^ ((text "      ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (paragraph (mode T ((text "Options:")))) ^^ ((text "\n")
) ^^ ((text "        ")) ^^ (description [
          (mode T ((text "-cnf"))), (mode T (((text "Forces the displayed formula to be in conjunctive normal form, only useful when ") ^^ (cmd (mode T ((text "-H")))) ^^ ((text " is set.")))));
          (mode T ((text "-D"))), (mode T ((text "Displays the formula in DIMACS cnf format (default)")));
          (mode T ((text "-H"))), (mode T ((text "Displays the formula in a human readable format")));
        ]) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Vim syntax file")))) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (vim_file) ^^ ((par)

) ^^ ((text "      ")) ^^ (bigskip) ^^ ((text "\n")
) ^^ ((text "      Copy the file into ")) ^^ (file (mode T ((text "~/.vim/syntax/")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      You can use modelines to force the syntax (see examples),")) ^^ ((text "\n")
) ^^ ((text "      you just have to add as the last line of your file:")) ^^ ((par)

) ^^ ((text "      ")) ^^ (cmd (mode T ((text "// vim:ft=regstab")))) ^^ ((par)

) ^^ ((text "    ")) ^^ (subsection (mode T ((text "Man pages")))) ^^ ((text "\n")
) ^^ ((text "      Short man pages for quick recall are available in the directory ")) ^^ (file (mode T ((text "man")))) ^^ ((text ".")) ^^ ((text "\n")
) ^^ ((text "      If you do not wish to install RegSTAB you can access them with ")) ^^ ((text "\n")
) ^^ ((text "      ")) ^^ (cmd (mode T ((text "man -M man/ regstab")))) ^^ ((text " or ")) ^^ (cmd (mode T ((text "man -M man/ sch2cnf")))) ^^ ((text " when in the top directory.")) ^^ ((text "\n")
) ^^ ((text "      However the full documentation is the present file.")) ^^ ((par)

) ^^ ((text "  ")) ^^ (section (mode T ((text "Licence")))) ^^ ((text "\n")
) ^^ ((text "    Free domain.")) ^^ ((par)

) ^^ ((text "  ")) ^^ (bibliographystyle (mode T ((text "alpha")))) ^^ ((text "\n")
) ^^ ((text "  ")) ^^ (bibliography (mode T ((text "aravantinos")))) ^^ ((text "\n")
)))))


let _ = emit (document ~documentclass:`Article ~author ~title ~packages:[(mode T ((text "graphicx"))),(mode T ((text "")))] doc)

